## DESCRIPTION: R scripts and functions that can be used to reproduce the
## results in the paper
## Kosmidis (2013). Improved estimation in cumulative link models.
## AUTHOR: Dr Ioannis Kosmidis
## DATE: Jan 2013
## EMAIL: i.kosmidis@ucl.ac.uk
## WEB: http://www.ucl.ac.uk/~ucakiko


## This script should be run one for each of the values 1, 2, 3, 4, 5,
## 6, 7, 8, 9, 10 for whichseed. Then the script Section10_results.R
## can be used to reproduce the reported figures in Section 10 of the
## paper

### Requires the pscl package

## Set whichseed
whichseed <- 1

source("bpolr.R")

## Available seeds
seeds <- c(5882, 1655, 419, 4624, 1689,
           111, 1309, 9891, 8712, 1476)

## Set seed according to the value of whichseed
set.seed(seeds[whichseed])

## The admit data set from the pscl library
library(pscl)
data(admit)

## Standardize the gre results
standardize <- function(x) { (x - mean(x))/sd(x) }
admit <- within(admit, {
                gre.quant <- standardize(gre.quant)
                gre.verbal <- standardize(gre.verbal) })

## admit <- within(admit, {
##                 gre.quant <- gre.quant/100
##                 gre.verbal <- gre.verbal/100 })


## Set up a proportional odds model for performing the parametric
## botstrap study
fmML <- bpolr(score ~ gre.quant + gre.verbal + ap + pt + female,
              data = admit,
              link = "logit")
truth <- fmML$beta
nam <- names(truth)

## Simulation size
nsimu <- 10000

## fun is a list with elements c(on what, what to do, result name)
stripObject <- function(obj, keep,
                        fun = list()) {
  whatIsIn <- names(obj)
  newObj <- obj[match(keep, whatIsIn, nomatch = 0)]
  for (i in seq.int(length.out = length(fun))) {
    curFun <- fun[[i]]
    newObj[[curFun[3]]] <- do.call(curFun[2], newObj[curFun[1]])
    newObj[[curFun[1]]] <- NULL
  }
  newObj
}

stdCalc <- function(vcov) {
  if (inherits(vcov, "try-error")) NA else sqrt(diag(vcov))
}

keep <- c("beta", "alpha", "vcov",
          "fitted.values",
          "scores", "adjustedScores",
          "boundaryEstimates",
          "inadmissible",
          "convergence",
          "dataReshaped",
          "niter")
fun <- list(c("vcov", "stdCalc", "stderr"))


resML <- resBR <- resBC <- as.list(rep(0, nsimu))
for (i in 1:nsimu) {
  cat(i, ": ")
  tempDat <- simulate(fmML)
  resML[[i]] <- stripObject(bpolr(score ~ gre.quant + gre.verbal + ap + pt + female,
                              data = tempDat,
                              weights = tempDat[["(weights)"]], method = "ML"),
                        keep = keep, fun = fun)
  cat("ML: Done! ")
  resBR[[i]] <- stripObject(bpolr(score ~ gre.quant + gre.verbal + ap + pt + female,
                              data = tempDat,
                              weights = tempDat[["(weights)"]], method = "BR"),
                        keep = keep, fun = fun)
  cat("BR: Done! ")
  resBC[[i]] <- stripObject(bpolr(score ~ gre.quant + gre.verbal + ap + pt + female,
                              data = tempDat,
                              weights = tempDat[["(weights)"]], method = "BC"),
                        keep = keep, fun = fun)
  cat("BC: Done! ")
  cat("\n")
}

alphaML <- simplify2array(lapply(resML, function(x) x$alpha))
alphaBR <- simplify2array(lapply(resBR, function(x) x$alpha))
alphaBC <- simplify2array(lapply(resBC, function(x) x$alpha))

betasML <- simplify2array(lapply(resML, function(x) x$beta))
betasBR <- simplify2array(lapply(resBR, function(x) x$beta))
betasBC <- simplify2array(lapply(resBC, function(x) x$beta))

boundaryEstimatesML <- which(simplify2array(lapply(resML,
                             function(x) x$boundaryEstimates)))
boundaryEstimatesBR <- which(simplify2array(lapply(resBR,
                             function(x) x$boundaryEstimates)))
boundaryEstimatesBC <- which(simplify2array(lapply(resBC,
                             function(x) x$boundaryEstimates)))

inadmissibleML <- which(simplify2array(lapply(resML,
                        function(x) x$inadmissible)))
inadmissibleBR <- which(simplify2array(lapply(resBR,
                        function(x) x$inadmissible)))
inadmissibleBC <- which(simplify2array(lapply(resBC,
                        function(x) x$inadmissible)))

simulatedData <- lapply(resML, function(x)
                        x$dataReshaped)

stderrsML <- simplify2array(lapply(resML, function(x) x$stderr[nam]))
stderrsBC <- simplify2array(lapply(resBC, function(x) x$stderr[nam]))
stderrsBR <- simplify2array(lapply(resBR, function(x) x$stderr[nam]))

biasesML <- apply(betasML, 1, mean) - truth
biasesBC <- apply(betasBC, 1, mean) - truth
biasesBR <- apply(betasBR, 1, mean) - truth

msesML <- apply((betasML - truth)^2, 1, mean)
msesBC <- apply((betasBC - truth)^2, 1, mean)
msesBR <- apply((betasBR - truth)^2, 1, mean)


cover <- function(beta, stderr, level = 0.95, truth) {
  quantile <- qnorm(1/2 + level/2)
  low <- beta - quantile * stderr
  upp <- beta + quantile * stderr
  rowMeans((low < truth) & (upp > truth))
}

coverML <- cover(betasML, stderrsML, level = 0.95, truth = truth)
coverBC <- cover(betasBC, stderrsBC, level = 0.95, truth = truth)
coverBR <- cover(betasBR, stderrsBR, level = 0.95, truth = truth)

rm(resML, resBR, resBC)

save.image(paste0("admit_", whichseed, ".RData"))


