## DESCRIPTION: R scripts and functions that can be used to reproduce the
## results in the paper
## Kosmidis (2013). Improved estimation in cumulative link models.
## AUTHOR: Dr Ioannis Kosmidis
## DATE: Jan 2013
## EMAIL: i.kosmidis@ucl.ac.uk
## WEB: http://www.ucl.ac.uk/~ucakiko

## Update 28/02/2013: The original script that was distributed with
## the article was based on the ordinal package version 2012.01-19.
## The latest version of the ordinal package (version 2013.9-20) tries
## to detect infinite estimates and if these occur then it reports NA
## values for the estimated standard errors and the z values. For this
## reason the output on this script does not full correspond to the
## output in Table 2 reported in the paper. The results for the
## reduced-bias estimator are exactly the same. For getting the exact
## results for maximum likelihood as in Table 2 of the paper, please
## use version 2012.01-19 of the ordinal package which is available at
## http://cran.r-project.org/src/contrib/Archive/ordinal/

## The analysis of the Wine tasting data in Examples 1.1 and
## Section~11 of the paper
source("bpolr.R")
library(ordinal)
data(wine)


## Maximum likelihood estimates for the parameters of model (1) in the
## paper (Table~1)
cc <- clm.control(gradTol = 1e-10)
mClm <- clm(rating ~ contact, nominal = ~ temp, data = wine,
            control = cc)
## The sign for the nominal effects
## is the reverse of the one shown in Table 1 of the paper because
## clm estimates the model with linear predictor
## \alpha_s + \beta_s w_r - \theta z_r instead of model (1) which has
## \alpha_s - \beta_s w_r - \theta z_r
(mClmSummary <- summary(mClm))
## formula: rating ~ contact
## nominal: ~temp
## data:    wine

##  link  threshold nobs logLik AIC    niter max.grad cond.H
##  logit flexible  72   -84.90 187.81 37(0) 2.55e-14 NA

## Coefficients:
##            Estimate Std. Error z value Pr(>|z|)
## contactyes    1.465         NA      NA       NA

## Threshold coefficients:
##                 Estimate Std. Error z value
## 1|2.(Intercept)   -1.266         NA      NA
## 2|3.(Intercept)    1.104         NA      NA
## 3|4.(Intercept)    3.766         NA      NA
## 4|5.(Intercept)   40.597         NA      NA
## 1|2.tempwarm     -37.095         NA      NA
## 2|3.tempwarm      -2.153         NA      NA
## 3|4.tempwarm      -2.873         NA      NA
## 4|5.tempwarm     -38.250         NA      NA
## Warning message:
## In summary.clm(mClm) :
##   Variance-covariance matrix of the parameters is not defined


## Reduced-bias estimates for the parameters of model (1) in the paper
## (Table~5)
mBpolr <- bpolr(rating ~ contact | 0 | temp, data = wine, method = "BR")
(mBpolrSummary <- summary(mBpolr))
## Call:
## bpolr(formula = rating ~ contact | 0 | temp, data = wine, method = "BR")

## Maximum absolute adjusted score:
## [1] 2.463593e-09

## Iterations for bias-reduced fit:  7

## Coefficients:
##              Value Std. Error t value
## contactyes   1.397     0.4630   3.018
## 1|2.tempwarm 2.621     1.5240   1.720
## 2|3.tempwarm 2.050     0.5789   3.541
## 3|4.tempwarm 2.648     0.7546   3.510
## 4|5.tempwarm 2.961     1.4992   1.975

## Intercepts:
##      Value Std. Error t value
## 1|2 -1.195     0.4987  -2.396
## 2|3  1.055     0.4358   2.420
## 3|4  3.498     0.7388   4.734
## 4|5  5.196     1.4746   3.524

## Residual Deviance: 171.8988
## AIC: 189.8988



## Wald-test based on the reduced-bias estimator (Section 11) and
## adjusted score test (Section 12).

## The function testPO allows to perform Wald, Score and LR test for
## proportional odds.
testPO <- function(fitFull, fitNest, test = c("Wald", "Score", "LR")) {
  test <- match.arg(test)
  coefsNest <- coef(fitNest)
  coefsFull <- coef(fitFull)
  nparFull <- length(coefsFull)
  nparNest <- length(coefsNest)
  df <- nparFull - nparNest
  betas <- coef(fitFull, what = "beta")
  betaNames <- names(betas)
  nominalNames <- betaNames[grep("\\|", names(betas))]
  if (test == "LR") {
    stat <- deviance(fitNest) - deviance(fitFull)
  }
  if (length(nominalNames) > 1) {
    if (test == "Wald") {
      contrastMatrix <- t(contr.sum(length(nominalNames)))
      gammas <- betas[nominalNames]%*%t(contrastMatrix)
      vinvGammas <- solve(contrastMatrix%*%vcov(fitFull)[nominalNames, nominalNames]%*%t(contrastMatrix))
      waldStatistic <- c(gammas%*%vinvGammas%*%t(gammas))
      stat <- waldStatistic
    }
    if (test == "Score") {
      what <- match(nominalNames, names(coefsFull))
      restNames <- names(coef(fitFull))[-what]
      whatValue <- coefsNest[lapply(strsplit(nominalNames, "\\."),
                                    function(x) x[[2]])[[1]]]
      coefsFull[what] <- whatValue
      coefsFull[restNames] <- coefsNest[restNames]
      newFit <- update(fitFull, start = coefsFull, maxit = 0,
                       method = if (fitFull$method == "BC") "ML" else fitFull$method)
      if (any(fitted(newFit) < 0 | fitted(newFit) >1)) {
        scoreStatistic <- NA
      }
      else {
        grad <- if (fitFull$method == "ML") newFit$scores else newFit$adjustedScores
        scoreStatistic <- grad %*% newFit$vcov %*% grad
      }
      stat <- scoreStatistic
    }
    structure(c(stat, df), .Names = c("stat", "df"))
  }
  else {
    structure(c(0, 1), .Names = c("stat", "df"))
  }
}

## Nested model with beta1 = beta2 = beta3 = beta4
mBpolrNest <- bpolr(rating ~ contact + temp, data = wine, method = "BR")

## Wald statistic
(WaldBR <- testPO(mBpolr, mBpolrNest, test = "Wald"))
##      stat        df
## 0.7501942 3.0000000
1 - pchisq(WaldBR["stat"], WaldBR["df"])
## 0.861339

## Score statistic
(ScoreBR <- testPO(mBpolr, mBpolrNest, test = "Score"))
##      stat        df
## 0.9356693 3.0000000
1 - pchisq(ScoreBR["stat"], ScoreBR["df"])
##      stat
## 0.8168126

