## DESCRIPTION: R scripts and functions that can be used to reproduce the
## results in the paper
## Kosmidis (2013). Improved estimation in cumulative link models.
## AUTHOR: Dr Ioannis Kosmidis
## DATE: Jan 2013
## EMAIL: i.kosmidis@ucl.ac.uk
## WEB: http://www.ucl.ac.uk/~ucakiko


## Before running the current script, the script Section8_9.R
## needs to be run 3 times, with setting the type variable therein to
## "1c" in one run, to "2c" in the second run and to "3c" in the third
## run.

## The script here assumes that the current directory contains a
## directory named images which contains the results that the script
## Section8_9.R will produce and that cImages.R is available
## in the current directory.


rm(list = ls())
source("cImages.R")
source("bpolr.R")
library(MASS)
library(ordinal)
library(lattice)
library(latticeExtra)

## Extract the required objects from the images
wd <- paste0(getwd(), "/images")
imageNames <- c("compenu_2x4_3_3logit.RData",
                "compenu_2x4_5_5logit.RData",
                "compenu_2x4_10_10logit.RData")

cImages(imageNames, c("betasBR", "betasBC", "betasML", "betasEL",
                      "stderrsBR", "stderrsBC", "stderrsML", "stderrsEL",
                      "dat", "tots", "isInf",
                      "boundaryML", "boundaryBR", "boundaryBC"),
        working.directory = wd)
cImages(imageNames[1], c("ncat", "Y", "x"), working.directory = wd)

catInList("betasBR", ".image", 1:length(imageNames), delete = TRUE)
catInList("betasML", ".image", 1:length(imageNames), delete = TRUE)
catInList("betasBC", ".image", 1:length(imageNames), delete = TRUE)
catInList("betasEL", ".image", 1:length(imageNames), delete = TRUE)
catInList("stderrsBR", ".image", 1:length(imageNames), delete = TRUE)
catInList("stderrsML", ".image", 1:length(imageNames), delete = TRUE)
catInList("stderrsBC", ".image", 1:length(imageNames), delete = TRUE)
catInList("stderrsEL", ".image", 1:length(imageNames), delete = TRUE)
catInList("dat", ".image", 1:length(imageNames), delete = TRUE)
catInList("tots", ".image", 1:length(imageNames), delete = TRUE)
catInList("isInf", ".image", 1:length(imageNames), delete = TRUE)
cObjects("Y", ".image", 1, delete = TRUE)
cObjects("x", ".image", 1, delete = TRUE)
cObjects("ncat", ".image", 1, delete = TRUE)

## Parameter settings to be considered -- as in the paper
fac <- c(1, 2, 3, 5, 7)
aMat <- rbind(t(matrix(c(-1, 0, 1), 3, length(fac)))*fac)
betas <- seq(-6, 6, length = 50)
probsL <- as.list(rep(0, nrow(aMat)))
for (jj in seq_len(nrow(aMat))) {
  alphas <- aMat[jj, ]
  probsL[[jj]] <- matrix(0, ncat*2, length(betas))
  for (ii in seq_len(length(betas))) {
    cat(jj,":", ii, "\n")
    cumprobs <- rbind(0, cbind(plogis(alphas + betas[ii]/2),
                               plogis(alphas - betas[ii]/2)), 1)
    probs <- apply(cumprobs, 2, diff)
    probsL[[jj]][,ii] <- c(probs)
  }
}


## Probability settings plot
pp <- c(unlist(probsL))
groups <- rep(rep(1:length(betas), each = 8), length(fac))
xx <- rep(rep(1:8, length(betas)), length(fac))
by <- paste("e =", factor(rep(fac, each = length(betas)*8)))
subset <- which(rep(rep(!as.logical((1:length(betas))%%10), each = 8), length(fac)))
p1 <- xyplot(pp ~ xx| by, groups = groups, col = "grey",
             type = "l",
             layout = c(length(fac), 1),
             panel = function(x, ...) {
                 for (i in 1:8)
                   panel.abline(v = i, lty = 2, lwd = 0.5, col = "black")
                 for (i in seq(0, 1, 0.2))
                   panel.abline(h = i, lty = 2, lwd = 0.5, col = "black")
                 panel.xyplot(x, ...)
                 panel.abline(v = 4.5, lty = 3, col = "darkgrey")
             },
             scales = list(x = list(at = 1:8,
                                    labels = c(paste("-0.5", 1:4, sep = "|"),
                                               paste("0.5", 1:4, sep = "|")),
                                    rot = 60,
                                    cex = 0.7,
                                    alternating = TRUE)),
             between = list(x = 0.5),
             xlab = "Covariate value | Category",
             ylab = "Probability")
p2 <- xyplot(pp ~ xx| by, groups = groups, col = "darkgrey",
             type = "l", subset = subset, layout = c(length(fac), 1))
p3 <- p1 + as.layer(p2)
print(p3)


## Calculation of biases, mses
bias <- function(estimates, probs, truth) {
    sum(estimates*probs) - truth
}

mse <- function(estimates, probs, truth) {
    sum((estimates - truth)^2*probs)
}

## Which images are to be considered?
which <- c(1, 2, 3)
nDat <- length(which)


## Parameter settings to be considered
fac <- c(1, 2, 3, 5, 7)
aMat <- rbind(t(matrix(c(-1, 0, 1), 3, length(fac)))*fac)
betas <- seq(-6, 0, length = 50)


##  Calculation of biases, mses
biasesEL <- biasesBR <- biasesBC <- biasesML <-
  msesBR <- msesEL <- msesML <- msesBC <-
  probsInf <- as.list(rep(0, length(dat)))
for (datId in which) {
  curBiasesEL <- curBiasesBR <- curBiasesBC <- curBiasesML <-
  curMsesBR <- curMsesEL <- curMsesML <- curMsesBC <-
  matrix(0, length(betas), nrow(aMat))
  curProbsInf <- matrix(0, length(betas), nrow(aMat))
  curDat <- dat[[datId]]
  curTots <- tots[[datId]]
  curIsInf <- isInf[[datId]]
  curBetaBR <- betasBR[[datId]]
  curBetaML <- betasML[[datId]]
  curBetaEL <- betasEL[[datId]]
  curBetaBC <- betasBC[[datId]]
  curStderrBR <- stderrsBR[[datId]]
  curStderrML <- stderrsML[[datId]]
  curStderrEL <- stderrsEL[[datId]]
  curStderrBC <- stderrsBC[[datId]]
  for (jj in seq_len(nrow(aMat))) {
    alphas <- aMat[jj, ]
    datProbs <- rep(0, ncol(curDat))
    for (ii in seq_len(length(betas))) {
      cat("image:", datId, ":", jj,":", ii, "\n")
      cumprobs <- rbind(0, cbind(plogis(alphas + betas[ii]/2),
                                 plogis(alphas - betas[ii]/2)), 1)
      probs <- apply(cumprobs, 2, diff)
      for (i in 1:ncol(curDat)) {
        datProbs[i] <- dmultinom(curDat[1:ncat ,i], curTots[1], probs[,1]) *
            dmultinom(curDat[ncat + 1:ncat ,i], curTots[2], probs[,2])
      }
      curProbsInf[ii, jj] <- sum(datProbs*curIsInf)
      curBiasesBR[ii,jj] <- bias(curBetaBR, datProbs, betas[ii])
      curBiasesEL[ii,jj] <- bias(curBetaEL, datProbs, betas[ii])
      curBiasesML[ii,jj] <- bias(curBetaML[!curIsInf],
                                 datProbs[!curIsInf]/(1 - curProbsInf[ii, jj]),
                                 betas[ii])
      curBiasesBC[ii,jj] <- bias(curBetaBC[!curIsInf],
                                 datProbs[!curIsInf]/(1 - curProbsInf[ii, jj]),
                                 betas[ii])
      curMsesBR[ii,jj] <- mse(curBetaBR, datProbs, betas[ii])
      curMsesEL[ii,jj] <- mse(curBetaEL, datProbs, betas[ii])
      curMsesML[ii,jj] <- mse(curBetaML[!curIsInf],
                                 datProbs[!curIsInf]/(1 - curProbsInf[ii, jj]),
                                 betas[ii])
      curMsesBC[ii,jj] <- mse(curBetaBC[!curIsInf],
                                 datProbs[!curIsInf]/(1 - curProbsInf[ii, jj]),
                                 betas[ii])
    }
  }
  biasesEL[[datId]] <- curBiasesEL
  biasesML[[datId]] <- curBiasesML
  biasesBC[[datId]] <- curBiasesBC
  biasesBR[[datId]] <- curBiasesBR
  msesEL[[datId]] <- curMsesEL
  msesML[[datId]] <- curMsesML
  msesBC[[datId]] <- curMsesBC
  msesBR[[datId]] <- curMsesBR
  probsInf[[datId]] <- curProbsInf
}


## Plotting

## Probability of infinite MLEs plot
pInf <- unlist(probsInf[which])
bs <- rep(rep(betas, length(fac)), nDat)
as <- factor(rep(rep(1:length(fac), each = length(betas)), nDat))
levels(as) <- paste("e =", fac)
totals <- factor(rep(which, each = length(fac)*length(betas)),
                 ordered = TRUE,
                 levels = rev(which))
levels(totals) <- unlist(lapply(tots[rev(which)],
                                function(x)
                                paste("m", 1:2, " = ",  x,
                                      collapse =" | ", sep = "")))
#
plotInf <- xyplot(pInf ~ bs | as + totals,
                  type = "l", col = "black",
                  layout = c(length(fac), nDat),
                  panel = function(x, ...) {
                    panel.xyplot(x, ...)
                    panel.abline(v = 0, col = "darkgrey")
                  },
                  ylim = c(0, 1),
                  xlab = "",
                  ylab = expression(paste("Prob. of infinite MLE")))


## Bias and Mses plots for ML and BC
mses <- c(unlist(msesML[which]), unlist(msesBC[which]))
biases <- c(unlist(biasesML[which]), unlist(biasesBC[which]))
bs <- rep(rep(betas, length(fac)*nDat), 2)
method <- factor(rep(1:2, each = length(fac)*length(betas)*nDat))
levels(method) <- c("ML", "BC")
as <- factor(rep(rep(1:length(fac), each = length(betas)), nDat*2))
levels(as) <- paste("e =", fac)
totals <- factor(rep(rep(which, each = length(fac)*length(betas)), 2),
                 ordered = TRUE,
                 levels = rev(which))
levels(totals) <- unlist(lapply(tots[rev(which)],
                                function(x)
                                paste("m", 1:2, " = ",  x,
                                      collapse =" | ", sep = "")))
#
MLplotMses <- xyplot(mses ~  bs | as + totals,
                     groups = method,
                     layout = c(length(fac), nDat),
                     type = "l", col = c("black", "grey"),
                     key = list(space = "top", cex = 1.2,
                     border = FALSE,
                     lines = list(col = c("black", "grey"), lty = c(1, 1)),
                     text = list(lab = c(expression(hat(beta)),
                                              expression(tilde(beta)[BC]))),
                     columns = 2),
                     lty = c(1, 1),
                     scales = list(x = list(alternating = TRUE)),
                     panel = function(x, ...) {
                       panel.abline(v = 0, col = "darkgrey")
                       panel.xyplot(x, ...)
                     },
                     between = list(x = 0),
                     xlab = expression(beta),
                     ylab = "Conditional mean squared error")
#
MLplotBiases <- xyplot(biases ~  bs | as + totals,
                       groups = method,
                       layout = c(length(fac), nDat),
                       type = "l", col = c("black", "grey"),
                       key = list(space = "top", cex = 1.2,
                       border = FALSE,
                       lines = list(col = c("black", "grey"),
                         lty = c(1, 1)),
                       text = list(lab = c(expression(hat(beta)),
                                     expression(tilde(beta)[BC]))),
                       columns = 2),
                       lty = c(1, 1),
                       scales = list(x = list(alternating = TRUE)),
                       panel = function(x, ...) {
                         panel.abline(0, 0, col = "darkgrey")
                         panel.xyplot(x, ...)
                       },
                       between = list(x = 0),
                       xlab = expression(beta),
                       ylab = "Conditional bias")


## Bias and Mses plots for BR and BC
mses <- c(unlist(msesBR[which]), unlist(msesEL[which]))
biases <- c(unlist(biasesBR[which]), unlist(biasesEL[which]))
bs <- rep(rep(betas, length(fac)*nDat), 2)
method <- factor(rep(1:2, each = length(fac)*length(betas)*nDat))
levels(method) <- c("BR", "EL")
as <- factor(rep(rep(1:length(fac), each = length(betas)), nDat*2))
levels(as) <- paste("e =", fac)
totals <- factor(rep(rep(which, each = length(fac)*length(betas)), 2),
                 ordered = TRUE,
                 levels = rev(which))
levels(totals) <- unlist(lapply(tots[rev(which)],
                                function(x)
                                paste("m", 1:2, " = ",  x,
                                      collapse =" | ", sep = "")))
BRplotMses <- xyplot(mses ~  bs | as + totals,
                   groups = method,
                   layout = c(length(fac), nDat),
                   type = "l", col = c("black", "grey"),
                   key = list(space = "top", cex = 1.2,
                     border = FALSE,
                     lines = list(col = c("black", "grey"), lty = c(1, 1)),
                     text = list(lab = c(expression(tilde(beta)[RB]),
                                   expression(hat(beta)[EL]))),
                     columns = 2),
                     lty = c(1, 1),
                     panel = function(x, ...) {
                       panel.abline(v = 0, col = "darkgrey")
                       panel.xyplot(x, ...)
                     },
                     between = list(x = 0),
                     xlab = expression(beta),
                     ylab = "Mean squared error")
#
BRplotBiases <- xyplot(biases ~  bs | as + totals,
                       groups = method, col = c("black", "grey"),
                       layout = c(length(fac), nDat),
                       type = "l",
                       key = list(space = "top", cex = 1.2,
                         border = FALSE,
                         lines = list(col = c("black", "grey"), lty = c(1, 1)),
                         text = list(lab = c(expression(tilde(beta)[RB]),
                                       expression(hat(beta)[EL]))),
                         columns = 2),
                       lty = c(1, 1),
                       panel = function(x, ...) {
                         panel.abline(0, 0, col = "darkgrey")
                         panel.xyplot(x, ...)
                       },
                       between = list(x = 0),
                       xlab = expression(beta),
                       ylab = "Bias")


## Coverage probabilities
cover <- function(estimates, stderrs, probs, truth, quantile = qnorm(0.975)) {
    lower <- estimates - quantile * stderrs
    upper <- estimates + quantile * stderrs
    inds <- which((stderrs > 200) | is.na(stderrs))
    lower[inds] <- -Inf
    upper[inds] <- +Inf
    sum(((lower < truth) & (upper > truth))*probs)
}

fac <- c(1, 2, 3, 5, 7)
aMat <- rbind(t(matrix(c(-1, 0, 1), 3, length(fac)))*fac)
betas <- seq(-10, 0, length = 100)


## Number of estimates per setting
which <- c(1, 2, 3)
nDat <- length(which)


##  Calculation of coverage probabilities
coverBR <- coverEL <- coverML <- coverBC <- as.list(rep(0, length(dat)))
for (datId in which) {
  curCoverBR <- curCoverEL <-
      curCoverML <- curCoverBC <-
          matrix(0, length(betas), nrow(aMat))
  curDat <- dat[[datId]]
  curTots <- tots[[datId]]
  curIsInf <- isInf[[datId]]
  curBetaBR <- betasBR[[datId]]
  curBetaML <- betasML[[datId]]
  curBetaEL <- betasEL[[datId]]
  curBetaBC <- betasBC[[datId]]
  curStderrBR <- stderrsBR[[datId]]
  curStderrML <- stderrsML[[datId]]
  curStderrEL <- stderrsEL[[datId]]
  curStderrBC <- stderrsBC[[datId]]
  for (jj in seq_len(nrow(aMat))) {
    alphas <- aMat[jj, ]
    datProbs <- rep(0, ncol(curDat))
    for (ii in seq_len(length(betas))) {
      cat("image:", datId, ":", jj,":", ii, "\n")
      cumprobs <- rbind(0, cbind(plogis(alphas + betas[ii]/2),
                                 plogis(alphas - betas[ii]/2)), 1)
      probs <- apply(cumprobs, 2, diff)
      for (i in 1:ncol(curDat)) {
          datProbs[i] <- dmultinom(curDat[1:ncat ,i], curTots[1], probs[,1]) *
              dmultinom(curDat[ncat + 1:ncat ,i], curTots[2], probs[,2])
      }
      curCoverBR[ii, jj] <- cover(curBetaBR, curStderrBR, datProbs, truth = betas[ii])
      curCoverEL[ii, jj] <- cover(curBetaEL, curStderrEL, datProbs, truth = betas[ii])
      curCoverML[ii, jj] <- cover(curBetaML, curStderrML, datProbs, truth = betas[ii])
      curCoverBC[ii, jj] <- cover(curBetaBC, curStderrBC, datProbs, truth = betas[ii])
    }
  }
  coverEL[[datId]] <- curCoverEL
  coverML[[datId]] <- curCoverML
  coverBC[[datId]] <- curCoverBC
  coverBR[[datId]] <- curCoverBR
}


## Coverage plots for ML and BC
coverage <- c(unlist(coverML[which]), unlist(coverBC[which]))
bs <- rep(rep(betas, length(fac)*nDat), 2)
method <- factor(rep(1:2, each = length(fac)*length(betas)*nDat))
levels(method) <- c("ML", "BC")
as <- factor(rep(rep(1:length(fac), each = length(betas)), nDat*2))
levels(as) <- paste("e =", fac)
totals <- factor(rep(rep(which, each = length(fac)*length(betas)), 2),
                 ordered = TRUE,
                 levels = rev(which))
levels(totals) <- unlist(lapply(tots[rev(which)],
                                function(x)
                                paste("m", 1:2, " = ",  x,
                                      collapse =" | ", sep = "")))
MLplotCover <- xyplot(coverage ~  bs | as + totals,
                      groups = method,
                      layout = c(length(fac), nDat),
                      type = "l", lty = c(1, 1), col = c("black", "grey"),
                      key = list(space = "top", cex = 1.2,
                        border = FALSE,
                        lines = list(col = c("black", "grey"), lty = c(1, 1)),
                        text = list(lab = c(expression(hat(beta)),
                                      expression(tilde(beta)[BC]))),
                        columns = 2),
                      panel = function(x, ...) {
                        panel.abline(0.95, 0, col = "black", lty = 3)
                        panel.abline(v = 0, col = "darkgrey")
                        panel.abline(h = 1, col = "black", lty = 3)
                        panel.xyplot(x, ...)
                      },
                      between = list(x = 0),
                      xlab = expression(beta),
                      ylab = "Coverage of 95% Wald-type intervals",
                      ylim = c(0.5, 1.05))

## Coverage plots for BR and EL
coverage <- c(unlist(coverBR[which]), unlist(coverEL[which]))
bs <- rep(rep(betas, length(fac)*nDat), 2)
method <- factor(rep(1:2, each = length(fac)*length(betas)*nDat))
levels(method) <- c("BR", "EL")
as <- factor(rep(rep(1:length(fac), each = length(betas)), nDat*2))
levels(as) <- paste("e =", fac)
totals <- factor(rep(rep(which, each = length(fac)*length(betas)), 2),
                 ordered = TRUE,
                 levels = rev(which))
levels(totals) <- unlist(lapply(tots[rev(which)],
                                function(x)
                                paste("m", 1:2, " = ",  x,
                                      collapse =" | ", sep = "")))
BRplotCover <- xyplot(coverage ~  bs | as + totals,
                      groups = method,
                      layout = c(length(fac), nDat),
                      type = "l", lty = c(1, 1), col = c("black", "grey"),
                      key = list(space = "top", cex = 1.2,
                        border = FALSE,
                        lines = list(col = c("black", "grey"), lty = c(1, 1)),
                        text = list(lab = c(expression(tilde(beta)[RB]),
                                      expression(hat(beta)[EL]))),
                        columns = 2),
                      panel = function(x, ...) {
                        panel.abline(0.95, 0, col = "black", lty = 3)
                        panel.abline(v = 0, col = "darkgrey")
                        panel.abline(h = 1, col = "black", lty = 3)
                        panel.xyplot(x, ...)
                      },
                      between = list(x = 0),
                      xlab = expression(beta),
                      ylab = "Coverage of 95% Wald-type intervals",
                      ylim = c(0.5, 1.05))


## Output plots in pdf format
w <- 10
h <- 5
pdf("MLmses.pdf", width = w, height = h)
MLplotMses
dev.off()
pdf("MLbiases.pdf", width = w, height = h)
MLplotBiases
dev.off()
pdf("BRmses.pdf", width = w, height = h)
BRplotMses
dev.off()
pdf("BRbiases.pdf", width = w, height = h)
BRplotBiases
dev.off()
pdf("MLcover.pdf", width = w, height = h)
update(MLplotCover, ylim = c(0.7, 1.05))
dev.off()
pdf("BRcover.pdf", width = w, height = h)
update(BRplotCover, ylim = c(0.7, 1.05))
dev.off()
pdf("probinf.pdf", width = w, height = h)
plotInf
dev.off()
pdf("allprobs.pdf", width = 10, height = 5)
p3
dev.off()




